/* this is nmmusic
 * nmmusic produces music on the background of X applications
 * Written by dozo <wilberth@sci.kun.nl> December 2001
 * GPL of course */
#include <X11/Intrinsic.h>
#include <string.h>
#include <stdio.h>
#include <math.h> //for pow
#include "nmmusic.h"
#define DEBUG 1
/* X support speakers in the range 20-20 000 Hz
 * I know the PC speaker almost supports this 
 * I don't know about other Xservers
 * don't expect much, on many xserver (xwin32) it doesn't do much
  4              5              6           octave
	C : 261.63     C : 523.25     C : 1046.50
	C#: 277.18     C#: 554.36     C#: 1108.73
	D : 293.66     D : 587.33     D : 1174.66
	Eb: 311.13     Eb: 622.25     Eb: 1244.51
	E : 329.63     E : 659.26     E : 1318.51
	F : 349.23     F : 698.46     F : 1396.91
	F#: 369.99     F#: 739.99     F#: 1479.98
	G : 392.00     G : 783.99     G : 1567.98
	G#: 415.30     G#: 830.61     G#: 1661.22
	A : 440.00     A : 880.00     A : 1760.00
	Bb: 466.16     Bb: 932.93     Bb: 1864.66
	B : 493.88     B : 987.77     B : 1975.53
*/
void musicCB(XtPointer client_data=NULL, XtIntervalId* id=NULL, 
		Display* display=0, XtAppContext app=NULL, const int* pitch=NULL, const int* duration=NULL, int tones=0){
	static XKeyboardControl kc;
	static int i=0;
	static int statTones;
	static const int* statDuration;
	static const int* statPitch;
	static Display* statDisplay;
	static XtAppContext statApp;
	if(client_data==0){
		statTones=tones;
		statDuration=duration;
		statPitch=pitch;
		statDisplay=display;
		statApp=app;
		i=0;
		if(tones==0){
			kc.bell_duration=-1; //restore default
			kc.bell_pitch=-1; //restore default
			XChangeKeyboardControl(statDisplay, KBBellPitch | KBBellDuration, &kc);
			return;
		}
	}
	if(i==statTones)i=0;
	if(DEBUG)printf("nmmusic: %2d %2d pitch: %5d duration: %4d\n", statTones, i, statPitch[i], statDuration[i]);
	kc.bell_duration=statDuration[i]/2;
	kc.bell_pitch=statPitch[i];
	XChangeKeyboardControl(statDisplay, KBBellPitch | KBBellDuration, &kc);
	if(statPitch[i])XBell(statDisplay, 100);
	XtAppAddTimeOut(statApp, statDuration[i++], (void (*)(void *, XtIntervalId *))musicCB, (void *)1); //ms, functie, client_data
}
int nmmusic(Display* display, XtAppContext app, const char* line, int ms){
	// 3A1 == 1/8 length octave 3 a (440 Hz)
	// 4B2 == 1/4 length octave 4 b (988 Hz)
	// 4Bb2 == ...
	if(!strcmp(line, "pink panther"))line="0P6 4D#2 4E2 0P4 4F#2 4G2 0P4 4D#2 4E2 4F#2 4G2 5C2\
 4B2 4E2 4G2 4B2 4A#8 4A2 4G2 4E2 4D2 4E8 0P6 4D#2\
 4E2 0P4 4F#2 4G2 0P4 4D#2 4E2 4F#2 4G2 5C2 4B2 4G2 4B2 5E2\
 5D#8  0P6 4D#2 4E2 0P4 4F#2 4G2 0P4 4D2 4E2 4F#2 4G2 5C2\
 4B2 4E2 4G2 4B2 4A#8 4A2 4G2 4E2 4D2 4E8";
	else if(!strcmp(line, "elise"))line="5E2 5D#2 5E2 5D#2 5E2 4B2 5D2 5C2 4A2 0P2"; 
	//else if(!strcmp(line, "donau"))line="
	const char* p=line;
	int tones=strlen(line)/3; //safe
	int* pitch=new int[tones];
	int* duration=new int[tones];
	int i=0; int octave, d; char l[10];
	int t; tones=0;
	while(1==sscanf(p, "%s", l)){
		octave=l[0]-48;
		switch(l[1]){
			case 'C':
			case 'c': t=0; break;
			case 'D':
			case 'd': t=2; break;
			case 'E':
			case 'e': t=4; break;
			case 'F':
			case 'f': t=5; break;
			case 'G':
			case 'g': t=7; break;
			case 'A':
			case 'a': t=9; break;
			case 'B':
			case 'b': t=11; break;
			case 'P':
			case 'p': t=99;
		}
		if(strlen(l)==4) {
			p+=4;
			d=l[3]-48;
			if(l[2]=='#')t++;
			else if(l[2]=='b')t--;
			else fprintf (stderr, "could not parse music string\n");
		} else if(strlen(l)==3) {
			p+=3;
			d=l[2]-48;
		} else {
			if(strlen(l)>0)p++;
			fprintf (stderr, "could not parse music string\n");
		}
		duration[tones]=d*ms/2;
		pitch[tones]=(int)(440*pow(2, (octave-4)+(double)(t-9)/12));
		if(t==99)pitch[tones]=0;
		while (p[0]==' ')p++;
		tones++;
	}
	musicCB(NULL, NULL, display, app, pitch, duration, tones);
	return 0;
}
